function gui = cgui(gui,style,label,content,tag,cb_handle,varargin)
%CGUI  cgui helps construct graphical user interface elements
% **********************************************************************
% *                   Part of the SNL NuMAD Toolbox                    *
% * Developed by Sandia National Laboratories Wind Energy Technologies *
% *             See license.txt for disclaimer information             *
% **********************************************************************
%   gui = cgui(gui,style,label,content,tag,cb_handle,varargin)
%   This function automates (to some extent) the positioning of gui
%   elements and provides a consistent layout
%
%       gui = structure to keep track of layout information
%     style = type of uicontrol, or other CGUI operations
%     label = text label to the left of a uicontrol
%   content = initial content of a uicontrol, or options in a dropdown menu
%       tag = tag which identifies the uicontrol
% cb_handle = function handle of the uicontrol's callback
%  varargin = non-standard arguments for various elements
%

% jcb: this function evolved as new gui elements were needed and a complete
% rewrite of this code is badly needed to further streamline gui creation

% calculate position of next item
% C.vpos = C.vpos + C.height + C.padding;
% pos.label = [C.hpos (P.height-C.vpos-2) C.width(1) C.height];
% pos.content = [C.hpos+C.width(1) (P.height-C.vpos) C.width(2) C.height];

labelVOffset = 5;

% the 'style' determines what sort of elements are created
switch style
    case 'init'
        gui.bgcolor = get(0,'defaultUicontrolBackgroundColor');

        gui.fig.h = [];
        gui.fig.xy = [0 0];
        
        %gui.panel.hpad = 10;
        %gui.panel.vpad = 10;
        
        gui.ctrl.hpad = 5;
        gui.ctrl.vpad = 2;
        
    case 'figure'
        %Figure - create new if necessary
        gui.fig.h = findobj('tag',tag);      % look for tag in current figures
        if isempty(gui.fig.h)
            % tag not found, create new
            gui.fig.h = figure('tag',tag,'integerhandle','off','Visible','off');
        else
            % tag found, clear figure
            clf(gui.fig.h);
        end
        %guidata(gui.fig.h,gui);  % save data structure in the figure guidata
        gui.fig.width=content(1);  
        gui.fig.height=content(2);
        set(gui.fig.h,...
            'Name',label,...
            'Visible','on',...
            'NextPlot','new',... %causes plot commands to ignore this figure
            'Resize','off',...
            'Units','pixels',...    % pixels | characters
            'Menubar','none',...
            'NumberTitle','off',...
            'Color',gui.bgcolor,...
            'Position',[gui.fig.xy(1) gui.fig.xy(2) gui.fig.width gui.fig.height]);
        %movegui(gui.fig.h,'onscreen');
        %gui.panel.hpos = gui.panel.hpad;
        %gui.panel.vpos = gui.panel.vpad;
        
    case 'axes'
        gui.ax.h = axes('Units',label,...
                        'Position',content);
        set(gui.ax.h,'Units','normalized');
        axis equal on;
        
    case 'panel'
        gui.panel.hpos = content(1);
        gui.panel.vpos = content(2);
        gui.panel.width=content(3);
        gui.panel.height=content(4);
        %gui.panel.vpos = gui.panel.vpos - gui.panel.height;
        gui.panel.h = uipanel(gui.fig.h,...
            'Title',label,...
            'Units','pixels',...
            'Position',[gui.panel.hpos gui.panel.vpos gui.panel.width gui.panel.height]);
        gui.ctrl.hpos = gui.ctrl.hpad;
        gui.ctrl.vpos = gui.panel.height - 20;
    
    case 'rowHeight'
        gui.ctrl.height = content;
        
    case 'colEdges'
        gui.ctrl.cedg = content;
        
    case 'vpos'
        switch label
            case 'top'
                gui.ctrl.vpos = gui.panel.height - 20;
            case 'previous'
                gui.ctrl.vpos = gui.ctrl.vpos + gui.ctrl.height + gui.ctrl.vpad;
            case 'next'
                gui.ctrl.vpos = gui.ctrl.vpos - gui.ctrl.height - gui.ctrl.vpad;  
            case 'value'
                gui.ctrl.vpos = content;
        end     
        
    case 'input'
        gui.ctrl.vpos = gui.ctrl.vpos - gui.ctrl.height - gui.ctrl.vpad;
        pos.label = [gui.ctrl.cedg(1),...
                     gui.ctrl.vpos - labelVOffset,...
                    (gui.ctrl.cedg(2)-gui.ctrl.cedg(1)-gui.ctrl.hpad),...
                     gui.ctrl.height];
        pos.content = [gui.ctrl.cedg(2),...
                       gui.ctrl.vpos,...
                      (gui.ctrl.cedg(3)-gui.ctrl.cedg(2)-gui.ctrl.hpad),...
                       gui.ctrl.height];
        uicontrol(gui.panel.h,...
            'Style','text',...
            'String',label,...
            'HorizontalAlignment','left',...
            'Position',pos.label);
        gui.ctrl.h = uicontrol(gui.panel.h,...
            'Style','edit',...
            'String',content,...
            'Tag',tag,...
            'BackgroundColor','white',...
            'Position',pos.content,...
            'Callback',cb_handle);
        
    case 'slideredit'
        gui.ctrl.vpos = gui.ctrl.vpos - gui.ctrl.height - gui.ctrl.vpad;
        pos.slider = [gui.ctrl.cedg(1),...
                     gui.ctrl.vpos+2,...
                    (gui.ctrl.cedg(2)-gui.ctrl.cedg(1)-gui.ctrl.hpad),...
                     gui.ctrl.height-4];
        pos.edit = [gui.ctrl.cedg(2),...
                       gui.ctrl.vpos,...
                      (gui.ctrl.cedg(3)-gui.ctrl.cedg(2)-gui.ctrl.hpad),...
                       gui.ctrl.height];
        uicontrol(gui.panel.h,...
            'Style','slider',...
            'Value',content,...
            'Position',pos.slider);
        gui.ctrl.h = uicontrol(gui.panel.h,...
            'Style','edit',...
            'String',content,...
            'Tag',tag,...
            'BackgroundColor','white',...
            'Position',pos.edit,...
            'Callback',cb_handle);
        
    case 'label'
        gui.ctrl.vpos = gui.ctrl.vpos - gui.ctrl.height - gui.ctrl.vpad;
        pos.label = [gui.ctrl.cedg(1),...
                     gui.ctrl.vpos - labelVOffset,...
                    (gui.ctrl.cedg(2)-gui.ctrl.cedg(1)-gui.ctrl.hpad),...
                     gui.ctrl.height];
        uicontrol(gui.panel.h,...
            'Style','text',...
            'String',label,...
            'HorizontalAlignment','left',...
            'Position',pos.label);
        
    case 'display'
        gui.ctrl.vpos = gui.ctrl.vpos - gui.ctrl.height - gui.ctrl.vpad;
        pos.label = [gui.ctrl.cedg(1),...
                     gui.ctrl.vpos - labelVOffset,...
                    (gui.ctrl.cedg(2)-gui.ctrl.cedg(1)-gui.ctrl.hpad),...
                     gui.ctrl.height];
        pos.content = [gui.ctrl.cedg(2),...
                       gui.ctrl.vpos - labelVOffset,...
                      (gui.ctrl.cedg(3)-gui.ctrl.cedg(2)-gui.ctrl.hpad),...
                       gui.ctrl.height];
        uicontrol(gui.panel.h,...
            'Style','text',...
            'String',label,...
            'HorizontalAlignment','left',...
            'Position',pos.label);
        gui.ctrl.h = uicontrol(gui.panel.h,...
            'Style','text',...
            'String',content,...
            'Tag',tag,...
            'Position',pos.content,...
            'Callback',cb_handle);    
        
    case 'textbox'
        gui.ctrl.vpos = gui.ctrl.vpos - gui.ctrl.height - gui.ctrl.vpad;
        pos.label = [gui.ctrl.cedg(1),...
                     gui.ctrl.vpos - labelVOffset,...
                    (gui.ctrl.cedg(2)-gui.ctrl.cedg(1)-gui.ctrl.hpad),...
                     gui.ctrl.height];
        pos.content = [gui.ctrl.cedg(2),...
                       gui.ctrl.vpos,...
                      (gui.ctrl.cedg(3)-gui.ctrl.cedg(2)-gui.ctrl.hpad),...
                       gui.ctrl.height];
        uicontrol(gui.panel.h,...
            'Style','text',...
            'String',label,...
            'HorizontalAlignment','left',...
            'Position',pos.label);
        gui.ctrl.h = uicontrol(gui.panel.h,...
            'Style','edit',...
            'Max',2,...  % setting Max=2 Min=0 allows multiply lines of text
            'Min',0,...
            'HorizontalAlignment','left',...
            'String',content,...
            'Tag',tag,...
            'BackgroundColor','white',...
            'Position',pos.content,...
            'Callback',cb_handle);    

    case 'num'
        gui.ctrl.vpos = gui.ctrl.vpos - gui.ctrl.height - gui.ctrl.vpad;
        pos.label = [gui.ctrl.cedg(1),...
                     gui.ctrl.vpos - labelVOffset,...
                    (gui.ctrl.cedg(2)-gui.ctrl.cedg(1)-gui.ctrl.hpad),...
                     gui.ctrl.height];
        pos.content = [gui.ctrl.cedg(2),...
                       gui.ctrl.vpos,...
                      (gui.ctrl.cedg(3)-gui.ctrl.cedg(2)-gui.ctrl.hpad),...
                       gui.ctrl.height];
        uicontrol(gui.panel.h,...
            'Style','text',...
            'String',label,...
            'HorizontalAlignment','left',...
            'Position',pos.label);
        gui.ctrl.h = uicontrol(gui.panel.h,...
            'Style','edit',...
            'String',content,...
            'Tag',tag,...
            'BackgroundColor','white',...
            'Position',pos.content,...
            'Callback',cb_handle);        
        
    case 'dnum'
        units = varargin{1};
        currentunits = 1;  % choose which units to start with
        gui.ctrl.vpos = gui.ctrl.vpos - gui.ctrl.height - gui.ctrl.vpad;
        pos.label = [gui.ctrl.cedg(1),...
                     gui.ctrl.vpos - labelVOffset,...
                    (gui.ctrl.cedg(2)-gui.ctrl.cedg(1)-gui.ctrl.hpad),...
                     gui.ctrl.height];
        pos.content = [gui.ctrl.cedg(2),...
                       gui.ctrl.vpos,...
                      (gui.ctrl.cedg(3)-gui.ctrl.cedg(2)-gui.ctrl.hpad),...
                       gui.ctrl.height];
        pos.units = [gui.ctrl.cedg(3),...
                     gui.ctrl.vpos,...
                    (gui.ctrl.cedg(4)-gui.ctrl.cedg(3)-gui.ctrl.hpad),...
                     gui.ctrl.height];
        uicontrol(gui.panel.h,...
            'Style','text',...
            'String',label,...
            'HorizontalAlignment','left',...
            'Position',pos.label);
        gui.ctrl.h = uicontrol(gui.panel.h,...
            'Style','edit',...
            'String',content * units{2}(currentunits),...
            'Tag',tag,...
            'BackgroundColor','white',...
            'Position',pos.content,...
            'Callback',cb_handle);
        uicontrol(gui.panel.h,...
            'Style','popupmenu',...
            'String',units{1},...
            'Value',currentunits,...
            'Tag',[tag '_units'],...
            'BackgroundColor','white',...
            'Position',pos.units,...
            'Userdata',units{2},...
            'Callback',cb_handle);
        % use the 'edit' control's value to store its units (the index
        % within the list of units)
        set(gui.ctrl.h,'Value',1);
        
    case 'select'
        gui.ctrl.vpos = gui.ctrl.vpos - gui.ctrl.height - gui.ctrl.vpad;
        pos.label = [gui.ctrl.cedg(1),...
                     gui.ctrl.vpos - labelVOffset,...
                    (gui.ctrl.cedg(2)-gui.ctrl.cedg(1)-gui.ctrl.hpad),...
                     gui.ctrl.height];
        pos.content = [gui.ctrl.cedg(2),...
                       gui.ctrl.vpos,...
                      (gui.ctrl.cedg(3)-gui.ctrl.cedg(2)-gui.ctrl.hpad),...
                       gui.ctrl.height];
        uicontrol(gui.panel.h,...
            'Style','text',...
            'String',label,...
            'HorizontalAlignment','left',...
            'Position',pos.label);
        gui.ctrl.h = uicontrol(gui.panel.h,...
            'Style','popupmenu',...
            'String',content,...
            'Tag',tag,...
            'BackgroundColor','white',...
            'Position',pos.content,...
            'Callback',cb_handle);   
        
    case 'listbox'
        listboxheight = varargin{1};
        gui.ctrl.vpos = gui.ctrl.vpos - gui.ctrl.height - gui.ctrl.vpad;
        pos.label = [gui.ctrl.cedg(1),...
                     gui.ctrl.vpos - labelVOffset,...
                    (gui.ctrl.cedg(2)-gui.ctrl.cedg(1)-gui.ctrl.hpad),...
                     gui.ctrl.height];
        pos.content = [gui.ctrl.cedg(1),...
                     gui.ctrl.vpos - listboxheight,...
                    (gui.ctrl.cedg(2)-gui.ctrl.cedg(1)-gui.ctrl.hpad),...
                     listboxheight];
        uicontrol(gui.panel.h,...
            'Style','text',...
            'String',label,...
            'HorizontalAlignment','left',...
            'Position',pos.label);
        gui.ctrl.h = uicontrol(gui.panel.h,...
            'Style','listbox',...
            'String',content,...
            'Tag',tag,...
            'BackgroundColor','white',...
            'Position',pos.content,...
            'Min',0,'Max',100,...
            'Value',[],...
            'Callback',cb_handle); 

    case 'textbox2'
        textboxheight = varargin{1};
        gui.ctrl.vpos = gui.ctrl.vpos - gui.ctrl.height - gui.ctrl.vpad;
        pos.label = [gui.ctrl.cedg(1),...
                     gui.ctrl.vpos - labelVOffset,...
                    (gui.ctrl.cedg(2)-gui.ctrl.cedg(1)-gui.ctrl.hpad),...
                     gui.ctrl.height];
        pos.content = [gui.ctrl.cedg(1),...
                       gui.ctrl.vpos - textboxheight,...
                      (gui.ctrl.cedg(2)-gui.ctrl.cedg(1)-gui.ctrl.hpad),...
                       textboxheight];
        uicontrol(gui.panel.h,...
            'Style','text',...
            'String',label,...
            'HorizontalAlignment','left',...
            'Position',pos.label);
        gui.ctrl.h = uicontrol(gui.panel.h,...
            'Style','edit',...
            'Max',2,...  % setting Max=2 Min=0 allows multiply lines of text
            'Min',0,...
            'HorizontalAlignment','left',...
            'String',content,...
            'Tag',tag,...
            'BackgroundColor','white',...
            'Position',pos.content,...
            'Callback',cb_handle);    
        
    case 'push'
        gui.ctrl.vpos = gui.ctrl.vpos - gui.ctrl.height - gui.ctrl.vpad;
        pos.content = [gui.ctrl.cedg(1),...
                       gui.ctrl.vpos,...
                      (gui.ctrl.cedg(2)-gui.ctrl.cedg(1)-gui.ctrl.hpad),...
                       gui.ctrl.height];
        gui.ctrl.h = uicontrol(gui.panel.h,...
            'Style','pushbutton',...
            'String',label,...
            'Tag',tag,...
            'Position',pos.content,...
            'Callback',cb_handle);
        
    case 'checkbox'
        gui.ctrl.vpos = gui.ctrl.vpos - gui.ctrl.height - gui.ctrl.vpad;
        pos.content = [gui.ctrl.cedg(1),...
                     gui.ctrl.vpos - labelVOffset,...
                    (gui.ctrl.cedg(2)-gui.ctrl.cedg(1)-gui.ctrl.hpad),...
                     gui.ctrl.height];
        gui.ctrl.h = uicontrol(gui.panel.h,...
            'Style','checkbox',...
            'String',label,...
            'Value',content,...
            'Tag',tag,...
            'Position',pos.content,...
            'Callback',cb_handle);
end

end